@@ -54,7 +54,7 @@ static void print_options(char* prog) {
         "    --attach_project <URL> <key>   attach to a project\n"
         "    --check_all_logins             for idle detection, check remote logins too\n"
         "    --daemon                       run as daemon (Unix)\n"
-        "    --detach                       detach from console (Windows)\n"
+        "    --detach_console               detach from console (Windows)\n"
         "    --detach_project <URL>         detach from a project\n"
         "    --dir <path>                   use given dir as BOINC home\n"
         "    --exit_after_app_start N       exit N seconds after an app starts\n"
@@ -136,7 +136,7 @@ void CLIENT_STATE::parse_cmdline(int argc, char** argv) {
             check_all_logins = true;
         } else if (ARG(daemon)) {
             executing_as_daemon = true;
-        } else if (ARG(detach_phase_two)) {
+        } else if (ARG(detach_console) || ARG(detach_phase_two)) {
             detach_console = true;
         } else if (ARG(detach_project)) {
             if (i == argc-1) show_options = true;
@@ -296,8 +296,8 @@ void CLIENT_STATE::parse_env_vars() {
         }
     }
 
-	p = getenv("SOCKS_SERVER");
-	if (!p) p = getenv("SOCKS5_SERVER");
+    p = getenv("SOCKS_SERVER");
+    if (!p) p = getenv("SOCKS5_SERVER");
     if (p && strlen(p)) {
         parse_url(p, purl);
         env_var_proxy_info.present = true;
@@ -308,13 +308,13 @@ void CLIENT_STATE::parse_env_vars() {
         env_var_proxy_info.socks_server_port = purl.port;
     }
 
-	p = getenv("SOCKS5_USER");
-	if (!p) p = getenv("SOCKS_USER");
+    p = getenv("SOCKS5_USER");
+    if (!p) p = getenv("SOCKS_USER");
     if (p) {
         strcpy(env_var_proxy_info.socks5_user_name, p);
     }
 
-	p = getenv("SOCKS5_PASSWD");
+    p = getenv("SOCKS5_PASSWD");
     if (p) {
         strcpy(env_var_proxy_info.socks5_user_passwd, p);
     }
