@@ -46,83 +46,6 @@ using std::string;
 LOG_FLAGS log_flags;
 CONFIG config;
 
-LOG_FLAGS::LOG_FLAGS() {
-    init();
-}
-
-void LOG_FLAGS::init() {
-    memset(this, 0, sizeof(LOG_FLAGS));
-    // on by default (others are off by default)
-    //
-    task = true;
-    file_xfer = true;
-    sched_ops = true;
-}
-
-// Parse log flag preferences
-//
-int LOG_FLAGS::parse(XML_PARSER& xp) {
-    char tag[1024];
-    bool is_tag;
-
-    while (!xp.get(tag, sizeof(tag), is_tag)) {
-        if (!is_tag) {
-            msg_printf_notice(NULL, false,
-                "http://boinc.berkeley.edu/manager_links.php?target=notice&controlid=log_flags",
-                "%s: %s",
-                _("Unexpected text in cc_config.xml"),
-                tag
-            );
-            continue;
-        }
-        if (!strcmp(tag, "/log_flags")) return 0;
-        if (xp.parse_bool(tag, "file_xfer", file_xfer)) continue;
-        if (xp.parse_bool(tag, "sched_ops", sched_ops)) continue;
-        if (xp.parse_bool(tag, "task", task)) continue;
-
-        if (xp.parse_bool(tag, "app_msg_receive", app_msg_receive)) continue;
-        if (xp.parse_bool(tag, "app_msg_send", app_msg_send)) continue;
-        if (xp.parse_bool(tag, "benchmark_debug", benchmark_debug)) continue;
-        if (xp.parse_bool(tag, "checkpoint_debug", checkpoint_debug)) continue;
-        if (xp.parse_bool(tag, "coproc_debug", coproc_debug)) continue;
-        if (xp.parse_bool(tag, "cpu_sched", cpu_sched)) continue;
-        if (xp.parse_bool(tag, "cpu_sched_debug", cpu_sched_debug)) continue;
-        if (xp.parse_bool(tag, "cpu_sched_status", cpu_sched_status)) continue;
-        if (xp.parse_bool(tag, "dcf_debug", dcf_debug)) continue;
-        if (xp.parse_bool(tag, "debt_debug", debt_debug)) continue;
-        if (xp.parse_bool(tag, "std_debug", std_debug)) continue;
-        if (xp.parse_bool(tag, "file_xfer_debug", file_xfer_debug)) continue;
-        if (xp.parse_bool(tag, "gui_rpc_debug", gui_rpc_debug)) continue;
-        if (xp.parse_bool(tag, "heartbeat_debug", heartbeat_debug)) continue;
-        if (xp.parse_bool(tag, "http_debug", http_debug)) continue;
-        if (xp.parse_bool(tag, "http_xfer_debug", http_xfer_debug)) continue;
-        if (xp.parse_bool(tag, "mem_usage_debug", mem_usage_debug)) continue;
-        if (xp.parse_bool(tag, "network_status_debug", network_status_debug)) continue;
-        if (xp.parse_bool(tag, "poll_debug", poll_debug)) continue;
-        if (xp.parse_bool(tag, "proxy_debug", proxy_debug)) continue;
-        if (xp.parse_bool(tag, "rr_simulation", rr_simulation)) continue;
-        if (xp.parse_bool(tag, "sched_op_debug", sched_op_debug)) continue;
-        if (xp.parse_bool(tag, "scrsave_debug", scrsave_debug)) continue;
-        if (xp.parse_bool(tag, "slot_debug", slot_debug)) continue;
-        if (xp.parse_bool(tag, "state_debug", state_debug)) continue;
-        if (xp.parse_bool(tag, "statefile_debug", statefile_debug)) continue;
-        if (xp.parse_bool(tag, "task_debug", task_debug)) continue;
-        if (xp.parse_bool(tag, "time_debug", time_debug)) continue;
-        if (xp.parse_bool(tag, "unparsed_xml", unparsed_xml)) continue;
-        if (xp.parse_bool(tag, "work_fetch_debug", work_fetch_debug)) continue;
-        if (xp.parse_bool(tag, "notice_debug", notice_debug)) continue;
-
-        msg_printf_notice(NULL, false,
-            "http://boinc.berkeley.edu/manager_links.php?target=notice&controlid=log_flags",
-            "%s: <%s>",
-            _("Unrecognized tag in cc_config.xml"),
-            tag
-        );
-        xp.skip_unexpected(tag, true, "LOG_FLAGS::parse");
-    }
-    return ERR_XML_PARSE;
-}
-
 static void show_flag(char* buf, bool flag, const char* flag_name) {
     if (!flag) return;
     int n = (int)strlen(buf);
@@ -216,7 +139,7 @@ void CONFIG::show() {
     if (fetch_minimal_work) {
         msg_printf(NULL, MSG_INFO, "Config: fetch minimal work");
     }
-    show_gpu_ignore(ignore_cuda_dev, "NVIDIA");
+    show_gpu_ignore(ignore_nvidia_dev, "NVIDIA");
     show_gpu_ignore(ignore_ati_dev, "ATI");
     for (i=0; i<exclusive_apps.size(); i++) {
         msg_printf(NULL, MSG_INFO,
@@ -253,65 +176,11 @@ void CONFIG::show() {
     }
 }
 
-CONFIG::CONFIG() {
-    clear();
-}
-
-// this is called first thing by client
-//
-void CONFIG::clear() {
-    abort_jobs_on_exit = false;
-    allow_multiple_clients = false;
-    allow_remote_gui_rpc = false;
-    alt_platforms.clear();
-    client_version_check_url = "http://boinc.berkeley.edu/download.php?xml=1";
-    client_download_url = "http://boinc.berkeley.edu/download.php";
-    disallow_attach = false;
-    dont_check_file_sizes = false;
-    dont_contact_ref_site = false;
-    exclusive_apps.clear();
-    exclusive_gpu_apps.clear();
-    exit_after_finish = false;
-    exit_when_idle = false;
-    fetch_minimal_work = false;
-    force_auth = "default";
-    http_1_0 = false;
-    http_transfer_timeout = 300;
-    http_transfer_timeout_bps = 10;
-    ignore_cuda_dev.clear();
-    ignore_ati_dev.clear();
-    max_file_xfers = MAX_FILE_XFERS;
-    max_file_xfers_per_project = MAX_FILE_XFERS_PER_PROJECT;
-    max_stderr_file_size = 0;
-    max_stdout_file_size = 0;
-    max_tasks_reported = 0;
-    ncpus = -1;
-    network_test_url = "http://www.google.com/";
-    no_alt_platform = false;
-    no_gpus = false;
-    no_info_fetch = false;
-    no_priority_change = false;
-    os_random_only = false;
-    report_results_immediately = false;
-    run_apps_manually = false;
-    save_stats_days = 30;
-    simple_gui_only = false;
-    skip_cpu_benchmarks = false;
-    start_delay = 0;
-    stderr_head = false;
-    suppress_net_info = false;
-    unsigned_apps_ok = false;
-    use_all_gpus = false;
-    use_certs = false;
-    use_certs_only = false;
-    zero_debts = false;
-}
-
-int CONFIG::parse_options(XML_PARSER& xp) {
+int CONFIG::parse_options_client(XML_PARSER& xp) {
     char tag[1024], path[256];
     bool is_tag;
     string s;
-    int n;
+    int n, retval;
 
     //clear();
     // don't do this here because some options are set by cmdline args,
@@ -321,7 +190,7 @@ int CONFIG::parse_options(XML_PARSER& xp) {
     alt_platforms.clear();
     exclusive_apps.clear();
     exclusive_gpu_apps.clear();
-    ignore_cuda_dev.clear();
+    ignore_nvidia_dev.clear();
     ignore_ati_dev.clear();
 
     while (!xp.get(tag, sizeof(tag), is_tag)) {
@@ -352,6 +221,22 @@ int CONFIG::parse_options(XML_PARSER& xp) {
             downcase_string(client_version_check_url);
             continue;
         }
+        if (!strcmp(tag, "coproc")) {
+            COPROC c;
+            retval = c.parse(xp);
+            if (retval) {
+                msg_printf(0, MSG_USER_ALERT,
+                    "Can't parse <coproc> in cc_config.xml"
+                );
+            }
+            retval = coprocs.add(c);
+            if (retval) {
+                msg_printf(0, MSG_USER_ALERT,
+                    "Duplicate <coproc> in cc_config.xml"
+                );
+            }
+            continue;
+        }
         if (xp.parse_str(tag, "data_dir", path, sizeof(path))) {
             if (chdir(path)) {
                 perror("chdir");
@@ -387,10 +272,8 @@ int CONFIG::parse_options(XML_PARSER& xp) {
             continue;
         }
         if (xp.parse_bool(tag, "http_1_0", http_1_0)) continue;
-        if (xp.parse_int(tag, "http_transfer_timeout", http_transfer_timeout)) continue;
-        if (xp.parse_int(tag, "http_transfer_timeout_bps", http_transfer_timeout_bps)) continue;
-        if (xp.parse_int(tag, "ignore_cuda_dev", n)) {
-            ignore_cuda_dev.push_back(n);
+        if (xp.parse_int(tag, "ignore_cuda_dev", n)||xp.parse_int(tag, "ignore_nvidia_dev", n)) {
+            ignore_nvidia_dev.push_back(n);
             continue;
         }
         if (xp.parse_int(tag, "ignore_ati_dev", n)) {
@@ -414,7 +297,7 @@ int CONFIG::parse_options(XML_PARSER& xp) {
         if (xp.parse_bool(tag, "os_random_only", os_random_only)) continue;
 #ifndef SIM
         if (!strcmp(tag, "proxy_info")) {
-            int retval = config_proxy_info.parse_config(*xp.f);
+            retval = config_proxy_info.parse_config(*xp.f);
             if (retval) return retval;
             continue;
         }
@@ -444,7 +327,7 @@ int CONFIG::parse_options(XML_PARSER& xp) {
     return ERR_XML_PARSE;
 }
 
-int CONFIG::parse(FILE* f) {
+int CONFIG::parse_client(FILE* f) {
     char tag[256];
     MIOFILE mf;
     XML_PARSER xp(&mf);
@@ -496,15 +379,20 @@ int CONFIG::parse(FILE* f) {
     return ERR_XML_PARSE;
 }
 
-int read_config_file(bool init) {
-    FILE* f;
+int CONFIG::parse(FILE* f) {
+    MIOFILE mf;
+    mf.init_file(f);
 
+    return parse(mf, log_flags);
+}
+
+int read_config_file(bool init, const char* fname) {
     if (!init) {
-        msg_printf(NULL, MSG_INFO, "Re-reading cc_config.xml");
-        config.clear();
+        msg_printf(NULL, MSG_INFO, "Re-reading %s", fname);
+        config.defaults();
         log_flags.init();
     }
-    f = boinc_fopen(CONFIG_FILE, "r");
+    FILE* f = boinc_fopen(fname, "r");
     if (!f) return ERR_FOPEN;
     int retval = config.parse(f);
     fclose(f);
@@ -514,6 +402,17 @@ int read_config_file(bool init) {
         config.max_stdout_file_size, config.max_stderr_file_size
     );
 #endif
+    if (init) {
+        coprocs = config.config_coprocs;
+        config_proxy_info = config.proxy_info;
+        if (strlen(config.data_dir)) {
+#ifdef _WIN32
+            _chdir(config.data_dir);
+#else
+            chdir(config.data_dir);
+#endif
+        }
+    }
     return 0;
 }
 
