@@ -18,13 +18,16 @@
 #ifndef __RR_SIM__
 #define __RR_SIM__
 
+#include <vector>
+#include <deque>
+
 struct RESULT;
 
 struct RR_SIM_PROJECT_STATUS {
-        /// jobs currently running (in simulation)
     std::vector<RESULT*>active;
-        /// CPU jobs runnable but not running yet
-    std::vector<RESULT*>pending;
+        // jobs currently running (in simulation)
+    std::deque<RESULT*>pending;
+        // CPU jobs runnable but not running yet
 
     inline void clear() {
         active.clear();
