@@ -23,15 +23,14 @@
 
 #include <vector>
 
-//#define USE_REC
-#define REC_HALF_LIFE (10*86400)
-//#define REC_HALF_LIFE (1*86400)
+extern bool use_rec;
+#define USE_REC
+extern double rec_half_life;
 
+extern bool use_hyst_fetch;
 
-#define RSC_TYPE_ANY    0
-#define RSC_TYPE_CPU    1
-#define RSC_TYPE_CUDA   2
-#define RSC_TYPE_ATI    3
+#define RSC_TYPE_ANY    -1
+#define RSC_TYPE_CPU    0
 
 struct PROJECT;
 struct RESULT;
@@ -46,12 +45,12 @@ struct RSC_PROJECT_WORK_FETCH {
     // the following are persistent (saved in state file)
     double backoff_time;
     double backoff_interval;
-#ifndef USE_REC
+//#ifndef USE_REC
     double long_term_debt;
     double short_term_debt;
     double anticipated_debt;
         // short-term debt, adjusted by scheduled jobs
-#endif
+//#endif
 
     // the following used by debt accounting
     double secs_this_debt_interval;
@@ -86,7 +85,7 @@ struct RSC_PROJECT_WORK_FETCH {
         memset(this, 0, sizeof(*this));
     }
 
-#ifndef USE_REC
+//#ifndef USE_REC
     // whether this project should accumulate debt for this resource
     //
     bool debt_eligible(PROJECT*, RSC_WORK_FETCH&);
@@ -95,16 +94,16 @@ struct RSC_PROJECT_WORK_FETCH {
         long_term_debt = 0;
         short_term_debt = 0;
     }
-#endif
+//#endif
 
     inline void reset() {
         backoff_time = 0;
         backoff_interval = 0;
-#ifndef USE_REC
+//#ifndef USE_REC
         long_term_debt = 0;
         short_term_debt = 0;
         anticipated_debt = 0;
-#endif
+//#endif
     }
 
     bool may_have_work;
@@ -115,9 +114,9 @@ struct RSC_PROJECT_WORK_FETCH {
         backoff_time = 0;
         backoff_interval = 0;
     }
-#ifndef USE_REC
+//#ifndef USE_REC
     bool overworked();
-#endif
+//#endif
 };
 
 // estimate the time a resources will be saturated
@@ -197,6 +196,9 @@ struct RSC_WORK_FETCH {
         // instance count for jobs that miss deadline
     std::deque<RESULT*> pending;
     BUSY_TIME_ESTIMATOR busy_time_estimator;
+#ifdef SIM
+    double estimated_delay;
+#endif
 
     void init(int t, int n, double sp) {
         rsc_type = t;
@@ -219,12 +221,14 @@ struct RSC_WORK_FETCH {
     void accumulate_shortfall(double d_time);
     void update_saturated_time(double dt);
     void update_busy_time(double dur, double nused);
+    PROJECT* choose_project_hyst();
     PROJECT* choose_project(int);
+    void supplement(PROJECT*);
     RSC_PROJECT_WORK_FETCH& project_state(PROJECT*);
-#ifndef USE_REC
+//#ifndef USE_REC
     void update_long_term_debts();
     void update_short_term_debts();
-#endif
+//#endif
     void print_state(const char*);
     void clear_request();
     void set_request(PROJECT*, bool allow_overworked);
@@ -247,16 +251,16 @@ struct RSC_WORK_FETCH {
 // per project state
 //
 struct PROJECT_WORK_FETCH {
-#ifdef USE_REC
+//#ifdef USE_REC
     double rec;
         // recent estimated credit
     double rec_time;
         // when it was last updated
     double rec_temp;
         // temporary copy used during schedule_cpus()
-#else
+//#else
     double overall_debt;
-#endif
+//#endif
     bool can_fetch_work;
     bool compute_can_fetch_work(PROJECT*);
     bool has_runnable_jobs;
@@ -269,10 +273,10 @@ struct PROJECT_WORK_FETCH {
 // global work fetch state
 //
 struct WORK_FETCH {
-#ifndef USE_REC
+//#ifndef USE_REC
     void set_overall_debts();
     void zero_debts();
-#endif
+//#endif
     PROJECT* choose_project();
         // find a project to ask for work
     PROJECT* non_cpu_intensive_project_needing_work();
@@ -286,6 +290,7 @@ struct WORK_FETCH {
     );
     void set_initial_work_request();
     void set_all_requests(PROJECT*);
+    void set_all_requests_hyst(PROJECT*, int rsc_type);
     void print_state();
     void init();
     void rr_init();
@@ -295,15 +300,14 @@ struct WORK_FETCH {
     void request_string(char*);
 };
 
-extern RSC_WORK_FETCH cuda_work_fetch;
-extern RSC_WORK_FETCH ati_work_fetch;
-extern RSC_WORK_FETCH cpu_work_fetch;
+extern RSC_WORK_FETCH rsc_work_fetch[MAX_RSC];
 extern WORK_FETCH work_fetch;
 
-#ifdef USE_REC
+//#ifdef USE_REC
 void project_priority_init();
 double project_priority(PROJECT*);
-void adjust_rec_temp(RESULT*);
-#endif
+void adjust_rec_sched(RESULT*);
+void adjust_rec_work_fetch(RESULT*);
+//#endif
 
 #endif
