@@ -132,8 +132,9 @@ enum RPC_SELECTOR {
     RPC_SET_GLOBAL_PREFS_OVERRIDE,
     RPC_GET_GLOBAL_PREFS_OVERRIDE_STRUCT,
     RPC_SET_GLOBAL_PREFS_OVERRIDE_STRUCT,   // 50
-    RPC_SET_DEBTS,
     RPC_GET_NOTICES,
+    RPC_GET_CC_CONFIG,
+    RPC_SET_CC_CONFIG,
     NUM_RPC_SELECTORS
 };
 
@@ -337,9 +338,10 @@ public:
             { return RPC_Wait(RPC_GET_GLOBAL_PREFS_OVERRIDE_STRUCT, (void*)&arg1, (void*)&arg2); }
     int set_global_prefs_override_struct(GLOBAL_PREFS& arg1, GLOBAL_PREFS_MASK& arg2)
             { return RPC_Wait(RPC_SET_GLOBAL_PREFS_OVERRIDE_STRUCT, (void*)&arg1, (void*)&arg2); }
-    int set_debts(std::vector<PROJECT> arg1)
-            { return RPC_Wait(RPC_SET_DEBTS, (void*)&arg1); }
-    
+    int get_cc_config(CONFIG& arg1, LOG_FLAGS& arg2)
+            { return RPC_Wait(RPC_GET_CC_CONFIG, (void*)&arg1, (void*)&arg2); }
+    int set_cc_config(CONFIG& arg1, LOG_FLAGS& arg2)
+            { return RPC_Wait(RPC_SET_CC_CONFIG, (void*)&arg1, (void*)&arg2); }
 private:
     CMainDocument*              m_pDoc;
 };
