@@ -16,9 +16,12 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
+require_once("../inc/util.inc");
 require_once("../inc/forum.inc");
 require_once("../inc/image.inc"); // Avatar scaling
 
+check_get_args(array());
+
 if (post_str("account_key", true) != null) {
     $user = lookup_user_auth(post_str("account_key"));
     $rpc = true;
