@@ -16,9 +16,12 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
+require_once('../inc/util.inc');
 require_once('../inc/forum_db.inc');
 require_once('../inc/forum_banishment_vote.inc');
 
+check_get_args(array("action", "userid"));
+
 $config = get_config();
 
 $logged_in_user = get_logged_in_user();
