@@ -24,6 +24,8 @@ require_once('../inc/forum.inc');
 require_once('../inc/bbcode_html.inc');
 require_once('../inc/akismet.inc');
 
+check_get_args(array("thread", "sort", "filter", "post", "nowrap", "no_quote", "tnow", "ttok"));
+
 $logged_in_user = get_logged_in_user(true);
 BoincForumPrefs::lookup($logged_in_user);
 check_banished($logged_in_user);
@@ -36,6 +38,8 @@ $filter = get_str('filter', true);
 $content = post_str('content', true);
 $preview = post_str("preview", true);
 $parent_post_id = get_int('post', true);
+$nowrap = get_str("nowrap",true);
+
 $parent_post = null;
 if ($parent_post_id) {
     $parent_post = BoincPost::lookup_id($parent_post_id);
@@ -103,22 +107,23 @@ echo "<p>";
 
 if ($preview == tra("Preview")) {
     $options = new output_options;
-    echo "<div id=\"preview\">\n";
-    echo "<div class=\"header\">".tra("Preview")."</div>\n";
-    echo output_transform($content, $options);
-    echo "</div>\n";
+    echo "<h2>".tra("Preview")."</h2>\n";
+    echo "<div class=\"pm_preview\">"
+        .output_transform($content, $options)
+        ."</div>\n"
+    ;
 }
 
 start_forum_table(array(tra("Author"), tra("Message")));
 
 show_message_row($thread, $parent_post);
-show_posts($thread, $forum, $sort_style, $filter, $logged_in_user, true);
+show_posts($thread, $forum, $sort_style, $filter, $logged_in_user, $nowrap, true);
 end_table();
 
 page_tail();
 
 function show_message_row($thread, $parent_post) {
-    global $g_logged_in_user, $bbcode_html;
+    global $logged_in_user, $bbcode_html;
     global $content, $preview;
 
     $x1 = "Message:".html_info().post_warning();
@@ -133,15 +138,15 @@ function show_message_row($thread, $parent_post) {
     }
 
     $x2 .= " method=\"post\" name=\"post\" onsubmit=\"return checkForm(this)\">\n";
-    $x2 .= form_tokens($g_logged_in_user->authenticator);
-    $x2 .= $bbcode_html."<textarea name=\"content\" rows=\"18\" cols=\"80\">";        
+    $x2 .= form_tokens($logged_in_user->authenticator);
+    $x2 .= $bbcode_html."<textarea name=\"content\" rows=\"18\" cols=\"80\">";
     $no_quote = get_int("no_quote", true)==1;
     if ($preview) {
         $x2 .= htmlspecialchars($content);
     } else if (!$no_quote) {
-        if ($parent_post) $x2 .= quote_text(htmlspecialchars($parent_post->content), 80)."\n";
+        if ($parent_post) $x2 .= quote_text(htmlspecialchars($parent_post->content))."\n";
     }
-    if (!$g_logged_in_user->prefs->no_signature_by_default){
+    if (!$logged_in_user->prefs->no_signature_by_default){
         $enable_signature="checked=\"true\"";
     } else {
         $enable_signature="";
@@ -158,10 +163,10 @@ function show_message_row($thread, $parent_post) {
     row2($x1, $x2);
 }
 
-function quote_text($text, $cols = 0) {
-	/* $cols is depricated. */
+function quote_text($text) {
     $text = "[quote]" . $text . "[/quote]";
     return $text;
 }
-$cvs_version_tracker[]="\$Id: forum_reply.php 20318 2010-01-29 17:00:14Z davea $";
+
+$cvs_version_tracker[]="\$Id: forum_reply.php 23466 2011-04-26 21:38:22Z davea $";
 ?>
