@@ -21,9 +21,13 @@
 
 require_once('../inc/util.inc');
 require_once('../inc/forum.inc');
+require_once('../inc/news.inc');
+
+check_get_args(array("id", "sort", "nowrap", "filter"));
 
 $threadid = get_int('id');
 $sort_style = get_int('sort', true);
+$nowrap = get_str('nowrap', true);
 $filter = get_str('filter', true);
 
 if ($filter != "false"){
@@ -39,10 +43,16 @@ if ($logged_in_user) {
     $tokens = url_tokens($logged_in_user->authenticator);
 }
 
+if ($threadid < 1) {
+    error_page(tra("Invalid thread ID!"));
+}
+
 $thread = BoincThread::lookup_id($threadid);
 $forum = BoincForum::lookup_id($thread->forum);
 
-if (!$thread) error_page(tra("No thread with id %1. Please check the link and try again.", $threadid));
+if (!$thread) {
+    error_page("Bad thread ID");
+}
 
 if (!is_forum_visible_to_user($forum, $logged_in_user)) {
     if ($logged_in_user) {
@@ -89,6 +99,7 @@ if ($logged_in_user && $logged_in_user->prefs->jump_to_unread){
 } else {
     page_head($title);
 }
+
 echo "<link href=\"forum_forum.php?id=".$forum->id."\" rel=\"up\" title=\"".$forum->title."\">";
 
 $is_subscribed = $logged_in_user && BoincSubscription::lookup($logged_in_user->id, $thread->id);
@@ -112,20 +123,21 @@ if ($forum->parent_type == 0) {
             if ($thread->owner == $logged_in_user->id){
                 if ($thread->replies !=0) {
                     // Show a "this question has been answered" to the author
-                    echo "<div class=\"helpdesk_note\">
-                    <form action=\"forum_thread_status.php\"><input type=\"hidden\" name=\"id\" value=\"".$thread->id."\">
-                    <input type=\"submit\" value=\"".tra("My question was answered")."\">
-                    </form>
-                    " . tra("If your question has been adequately answered please click here to close it!") . "
-                    </div>";
+                    echo "<p>";
+                    show_button(
+                        "forum_thread_status.php?id=$thread->id&action=set",
+                        tra("My question was answered"),
+                        tra("Click here if your question has been adequately answered") 
+                    );
                 }
             } else {
-                // and a "I also got this question" to everyone else if they havent already told so
-                echo "<div class=\"helpdesk_note\">
-                <form action=\"forum_thread_vote.php\"><input type=\"hidden\" name=\"id\" value=\"".$thread->id."\">
-                <input type=\"submit\" value=\"".tra("I've also got this question")."\">
-                </form>
-                </div>";
+                // and a "I also got this question" to everyone else
+                echo "<p>";
+                show_button(
+                    "forum_thread_vote.php?id=$thread->id",
+                    tra("I've also got this question"),
+                    tra("I've also got this question")
+                );
             }
         }
     }
@@ -136,49 +148,112 @@ echo "
     <form action=\"forum_thread.php\">
     <table width=\"100%\" cellspacing=0 cellpadding=0>
     <tr class=\"forum_toplinks\">
-    <td><ul class=\"actionlist\">
+    <td>
 ";
 
 $reply_url = "";
-if (can_reply($thread, $forum, $logged_in_user)) {        
+if (can_reply($thread, $forum, $logged_in_user)) {
     $reply_url = "forum_reply.php?thread=".$thread->id."#input";
-    show_actionlist_button($reply_url, tra("Post to thread"), tra("Add a new message to this thread"));
+    show_button(
+        $reply_url,
+        tra("Post to thread"),
+        tra("Add a new message to this thread")
+    );
 }
 
 if ($is_subscribed) {
     $type = NOTIFY_SUBSCRIBED_POST;
-    BoincNotify::delete_aux("userid=$logged_in_user->id and type=$type and opaque=$thread->id");
+    BoincNotify::delete_aux(
+        "userid=$logged_in_user->id and type=$type and opaque=$thread->id"
+    );
     $url = "forum_subscribe.php?action=unsubscribe&thread=".$thread->id."$tokens";
-    show_actionlist_button($url, tra("Unsubscribe"), tra("You are subscribed to this thread.  Click here to unsubscribe."));
+    show_button(
+        $url,
+        tra("Unsubscribe"),
+        tra("You are subscribed to this thread.  Click here to unsubscribe.")
+    );
 } else {
     $url = "forum_subscribe.php?action=subscribe&thread=".$thread->id."$tokens";
-    show_actionlist_button($url, tra("Subscribe"), tra("Click to get email when there are new posts in this thread"));
+    show_button(
+        $url,
+        tra("Subscribe"),
+        tra("Click to get email when there are new posts in this thread")
+    );
 }
 
 //If the logged in user is moderator enable some extra features
 //
 if (is_moderator($logged_in_user, $forum)) {
     if ($thread->hidden){
-        show_actionlist_button("forum_moderate_thread_action.php?action=unhide&thread=".$thread->id."$tokens", tra("Unhide"), tra("Unhide this thread"));
+        show_button(
+            "forum_moderate_thread_action.php?action=unhide&thread=".$thread->id."$tokens",
+            tra("Unhide"),
+            tra("Unhide this thread")
+        );
     } else {
-        show_actionlist_button("forum_moderate_thread.php?action=hide&thread=".$thread->id, tra("Hide"), tra("Hide this thread"));
+        show_button(
+            "forum_moderate_thread.php?action=hide&thread=".$thread->id,
+            tra("Hide"),
+            tra("Hide this thread")
+        );
     }
     if ($thread->sticky){
-        show_actionlist_button("forum_moderate_thread_action.php?action=desticky&thread=".$thread->id."$tokens", tra("Make unsticky"), tra("Make this thread not sticky"));
+        show_button(
+            "forum_moderate_thread_action.php?action=desticky&thread=".$thread->id."$tokens",
+            tra("Make unsticky"),
+            tra("Make this thread not sticky")
+        );
     } else {
-        show_actionlist_button("forum_moderate_thread_action.php?action=sticky&thread=".$thread->id."$tokens", tra("Make sticky"), tra("Make this thread sticky"));
+        show_button(
+            "forum_moderate_thread_action.php?action=sticky&thread=".$thread->id."$tokens",
+            tra("Make sticky"),
+            tra("Make this thread sticky")
+        );
     }
     if ($thread->locked) {
-        show_actionlist_button("forum_moderate_thread_action.php?action=unlock&amp;thread=".$thread->id."$tokens", tra("Unlock"), tra("Unlock this thread"));
+        show_button(
+            "forum_moderate_thread_action.php?action=unlock&amp;thread=".$thread->id."$tokens",
+            tra("Unlock"),
+            tra("Unlock this thread")
+        );
     } else {
-        show_actionlist_button("forum_moderate_thread.php?action=lock&thread=".$thread->id."$tokens", tra("Lock"), tra("Lock this thread"));
+        show_button(
+            "forum_moderate_thread.php?action=lock&thread=".$thread->id."$tokens",
+            tra("Lock"),
+            tra("Lock this thread")
+        );
     }
     if ($forum->parent_type == 0) {
-        show_actionlist_button("forum_moderate_thread.php?action=move&thread=".$thread->id."$tokens", tra("Move"), tra("Move this thread to a different forum"));
+        show_button(
+            "forum_moderate_thread.php?action=move&thread=".$thread->id."$tokens",
+            tra("Move"),
+            tra("Move this thread to a different forum")
+        );
+    }
+    show_button(
+        "forum_moderate_thread.php?action=title&thread=".$thread->id."$tokens",
+        tra("Edit title"),
+        tra("Edit thread title")
+    );
+}
+
+// let admins decide whether a news item should be exported as notice
+//
+if (is_news_forum($forum) && $logged_in_user && ($logged_in_user->id == $thread->owner)) {
+    if ($thread->status) {
+        show_button(
+            "forum_thread_status.php?action=clear&id=$thread->id",
+            "Export",
+            "Export this news item as a Notice"
+        );
+    } else {
+        show_button(
+            "forum_thread_status.php?action=set&id=$thread->id",
+            "Don't export",
+            "Don't export this news item as a Notice"
+        );
     }
-    show_actionlist_button("forum_moderate_thread.php?action=title&thread=".$thread->id."$tokens", tra("Edit title"), tra("Edit thread title"));
 }
-echo "</ul>"; //End of action list
 
 // Display a box that allows the user to select sorting of the posts
 echo "</td><td align=\"right\">
@@ -192,12 +267,18 @@ echo "<input type=\"submit\" value=\"Sort\">
 // Here is where the actual thread begins.
 $headings = array(array(tra("Author"),"authorcol"), array(tra("Message"),""));
 
-start_forum_table($headings, "id=\"thread\" width=100%");
-show_posts($thread, $forum, $sort_style, $filter, $logged_in_user, true);
+start_forum_table($headings, "id=\"thread\" cellspacing=0");
+show_posts(
+    $thread, $forum, $sort_style, $filter, $logged_in_user, $nowrap, true
+);
 end_table();
 
 if ($reply_url) {
-    show_button($reply_url, tra("Post to thread"), tra("Add a new message to this thread"));
+    show_button(
+        $reply_url,
+        tra("Post to thread"),
+        tra("Add a new message to this thread")
+    );
 }
 
 echo "<p>";
@@ -213,5 +294,5 @@ case 1:
 $thread->update("views=views+1");
 
 page_tail();
-$cvs_version_tracker[]="\$Id: forum_thread.php 21003 2010-03-26 05:34:14Z boincadm $";
+$cvs_version_tracker[]="\$Id: forum_thread.php 23235 2011-03-14 23:56:14Z davea $";
 ?>
