@@ -20,6 +20,8 @@ require_once("../inc/db.inc");
 require_once("../inc/util.inc");
 require_once("../inc/user.inc");
 
+check_get_args(array());
+
 page_head("Forgot your account info?");
 
 echo "
@@ -32,7 +34,7 @@ You will be sent email instructions for resetting your password.
 
 start_table();
 echo "<form method=post action=mail_passwd.php>\n";
-row2("Email address","<input size=40 name=email_addr>");
+row2("Email address","<input type=\"text\" size=40 name=email_addr>");
 row2("", "<input type=submit value=OK>");
 echo "</form>";
 end_table();
@@ -75,7 +77,7 @@ update the email and password of your account.
 start_table();
 
 echo "<form action=login_action.php method=post>\n";
-row2("Log in with authenticator", "<input name=authenticator size=40>");
+row2("Log in with authenticator", "<input type=\"text\" name=authenticator size=40>");
 row2("Stay logged in on this computer",
     "<input type=checkbox name=send_cookie checked>"
 );
