@@ -20,19 +20,7 @@ require_once("../inc/db.inc");
 require_once("../inc/util.inc");
 require_once("../inc/host.inc");
 
-function fail($msg) {
-    echo "Error: $msg";
-    page_tail();
-    exit();
-}
-
-function get_host($hostid, $user) {
-    $host = lookup_host($hostid);
-    if (!$host || $host->userid != $user->id) {
-        fail("We have no record of that computer");
-    }
-    return $host;
-}
+check_get_args(array("hostid"));
 
 db_init();
 $user = get_logged_in_user();
@@ -40,6 +28,12 @@ $user = get_logged_in_user();
 page_head("Updating computer credit");
 
 $hostid = get_int("hostid");
+
+$host = lookup_host($hostid);
+if (!$host || $host->userid != $user->id) {
+    error_page("We have no record of that computer");
+}
+
 host_update_credit($hostid);
 echo "<br>Host credit updated";
 page_tail();
