@@ -22,6 +22,11 @@ require_once("../inc/pm.inc");
 require_once("../inc/forum.inc");
 require_once("../inc/akismet.inc");
 
+check_get_args(array("replyto", "deleted", "userid", "action", "sent", "id", "tnow", "ttok"));
+
+$replyto = get_int("replyto", true);
+$userid = get_int("userid", true);
+
 function show_block_link($userid) {
     echo " <a href=\"pm.php?action=block&amp;id=$userid\">";
     show_image(REPORT_POST_IMAGE, "Block messages from this user",  "Block user", REPORT_POST_IMAGE_HEIGHT);
@@ -62,13 +67,13 @@ function make_script() {
 //
 function do_inbox($logged_in_user) {
     page_head(tra("Private messages").": ".tra("Inbox"));
-    
+
     make_script();
     if (get_int("sent", true) == 1) {
         echo "<div class=\"notice\">".tra("Your message has been sent.")."</div>\n";
     }
     $options = get_output_options($logged_in_user);
-    
+
     BoincNotify::delete_aux("userid=$logged_in_user->id and type=".NOTIFY_PM);
 
     $msgs = BoincPrivateMessage::enum(
@@ -88,7 +93,7 @@ function do_inbox($logged_in_user) {
             $i++;
             $class = ($i%2)? "row0": "row1";
             echo "<tr class=$class>\n";
-            $checkbox = "<input type=\"checkbox\" name=\"pm_select[]\" value=\"".$msg->id."\">";
+            $checkbox = "<input type=checkbox name=pm_select_$msg->id>";
             if (!$msg->opened) {
                 $msg->update("opened=1");
             }
@@ -98,9 +103,8 @@ function do_inbox($logged_in_user) {
             echo "<br>".time_str($msg->date)."</td>\n";
             echo "<td valign=top>".output_transform($msg->content, $options)."<p>";
             $tokens = url_tokens($logged_in_user->authenticator);
-	    echo "<ul class=\"actionlist\">";
-            show_actionlist_button("pm.php?action=new&amp;replyto=$msg->id", tra("Reply"), "Reply to this message");
-            show_actionlist_button("pm.php?action=delete&amp;id=$msg->id&amp;$tokens", tra("Delete"), "Delete this message");
+            show_button("pm.php?action=new&amp;replyto=$msg->id", tra("Reply"), "Reply to this message");
+            show_button("pm.php?action=delete&amp;id=$msg->id&amp;$tokens", tra("Delete"), "Delete this message");
             echo "</ul></td></tr>\n";
         }
         echo "
@@ -128,7 +132,7 @@ function do_read($logged_in_user) {
     }
     page_head(tra("Private messages")." : ".$message->subject);
     pm_header();
-    
+
     $sender = BoincUser::lookup_id($message->senderid);
 
     start_table();
@@ -143,15 +147,16 @@ function do_read($logged_in_user) {
     echo " | <a href=\"pm.php?action=delete&amp;id=$id\">".tra("Delete")."</a>\n";
     echo " | <a href=\"pm.php?action=inbox\">".tra("Inbox")."</a>\n";
     end_table();
-    
+
     if ($message->opened == 0) {
         $message->update("opened=1");
     }
 }
 
 function do_new($logged_in_user) {
+    global $replyto, $userid;
     check_banished($logged_in_user);
-    pm_form();
+    pm_form($replyto, $userid);
 }
 
 function do_delete($logged_in_user) {
@@ -165,64 +170,65 @@ function do_delete($logged_in_user) {
 }
 
 function do_send($logged_in_user) {
+    global $replyto, $userid;
     check_banished($logged_in_user);
     check_tokens($logged_in_user->authenticator);
-    
+
     $to = sanitize_tags(post_str("to", true));
     $subject = post_str("subject", true);
     $content = post_str("content", true);
-    
+
     if (post_str("preview", true) == tra("Preview")) {
-        pm_form();
+        pm_form($replyto, $userid);
     }
     if (($to == null) || ($subject == null) || ($content == null)) {
-        pm_form(tra("You need to fill all fields to send a private message"));
+        pm_form($replyto, $userid, tra("You need to fill all fields to send a private message"));
     } else {
         if (!akismet_check($logged_in_user, $content)) {
-            pm_form("Your message was flagged as spam
+            pm_form($replyto, $userid, "Your message was flagged as spam
                 by the Akismet anti-spam system.
                 Please modify your text and try again."
             );
         }
         $to = str_replace(", ", ",", $to); // Filter out spaces after separator
         $users = explode(",", $to);
-        
+
         $userlist = array();
         $userids = array(); // To prevent from spamming a single user by adding it multiple times
-        
+
         foreach ($users as $username) {
             $user = explode(" ", $username);
             if (is_numeric($user[0])) { // user ID is gived
                 $userid = $user[0];
                 $user = lookup_user_id($userid);
                 if ($user == null) {
-                    pm_form(tra("Could not find user with id %1", $userid));
+                    pm_form($replyto, $userid, tra("Could not find user with id %1", $userid));
                 }
             } else {
                 $user = lookup_user_name($username);
                 if ($user == null) {
-                    pm_form(tra("Could not find user with username %1", $username));
+                    pm_form($replyto, $userid, tra("Could not find user with username %1", $username));
                 } elseif ($user == -1) { // Non-unique username
-                    pm_form(tra("%1 is not a unique username; you will have to use user ID", $username));
+                    pm_form($replyto, $userid, tra("%1 is not a unique username; you will have to use user ID", $username));
                 }
             }
             BoincForumPrefs::lookup($user);
             if (is_ignoring($user, $logged_in_user)) {
-                pm_form(tra("User %1 (ID: %2) is not accepting private messages from you.", $user->name, $user->id));
+                pm_form($replyto, $userid, tra("User %1 (ID: %2) is not accepting private messages from you.", $user->name, $user->id));
             }
             if ($userids[$user->id] == null) {
                 $userlist[] = $user;
                 $userids[$user->id] = true;
             }
         }
-        
+
         foreach ($userlist as $user) {
             if (!is_moderator($logged_in_user, null)) {
                 check_pm_count($logged_in_user->id);
             }
             pm_send($user, $subject, $content, true);
         }
-        
+
         Header("Location: pm.php?action=inbox&sent=1");
     }
 }
@@ -237,7 +243,7 @@ function do_block($logged_in_user) {
     echo "<div>".tra("Are you really sure you want to block user %1 from sending you private messages?", $user->name)."<br>\n";
     echo tra("Please note that you can only block a limited amount of users.")."</div>\n";
     echo "<div>".tra("Once the user has been blocked you can unblock it using forum preferences page.")."</div>\n";
-    
+
     echo "<form action=\"pm.php\" method=\"POST\">\n";
     echo form_tokens($logged_in_user->authenticator);
     echo "<input type=\"hidden\" name=\"action\" value=\"confirmedblock\">\n";
@@ -253,19 +259,23 @@ function do_confirmedblock($logged_in_user) {
     $blocked_user = BoincUser::lookup_id($id);
     if (!$blocked_user) error_page("no such user");
     add_ignored_user($logged_in_user, $blocked_user);
-    
+
     page_head(tra("User %1 blocked", $blocked_user->name));
-    
+
     echo "<div>".tra("User %1 has been blocked from sending you private messages.", $blocked_user->name)."\n";
     echo tra("To unblock, visit %1message board preferences%2", "<a href=\"edit_forum_preferences_form.php\">", "</a>")."</div>\n";
 }
 
 function do_delete_selected($logged_in_user) {
     check_tokens($logged_in_user->authenticator);
-    foreach ($_POST["pm_select"] as $id) {
-        $id = BoincDb::escape_string($id);
-        $msg = BoincPrivateMessage::lookup_id($id);
-        if ($msg && $msg->userid == $logged_in_user->id) {
+
+    $msgs = BoincPrivateMessage::enum(
+        "userid=$logged_in_user->id"
+    );
+    foreach($msgs as $msg) {
+        $x = "pm_select_$msg->id";
+        if ($_POST[$x]) {
+            $msg = BoincPrivateMessage::lookup_id($msg->id);
             $msg->delete();
         }
     }
