@@ -20,9 +20,11 @@ include_once("../inc/db.inc");
 include_once("../inc/util.inc");
 include_once("../inc/team.inc");
 
+check_get_args(array());
+
 page_head("Teams");
 
-echo "<p>".tra("%1 participants may form %2teams%3", PROJECT, "<b>", "</b>") ."
+echo "<p>".tra("%1 participants may form %2teams%3.", PROJECT, "<b>", "</b>") ."
     <p>
     ". tra("You may belong to only one team. You can join or quit a team at any time."). "
     <p>" .
