@@ -153,6 +153,8 @@ struct APP_INIT_DATA {
     char symstore[256];
     char acct_mgr_url[256];
     char* project_preferences;
+    int userid;
+    int teamid;
     int hostid;
         // project's DB ID for this host (NOT host CPID)
     char user_name[256];
@@ -160,6 +162,7 @@ struct APP_INIT_DATA {
     char project_dir[256];
     char boinc_dir[256];
     char wu_name[256];
+    char result_name[256];
     char authenticator[256];
     int slot;
     double user_total_credit;
