@@ -42,6 +42,10 @@
 #define HAVE_ALLOCA 1
 #endif 
 
+#ifdef __MINGW32__
+#define HAVE_STRCASECMP 1
+#endif
+
 #else
 
 // Under CYGWIN we need to include config.h first.
@@ -243,21 +247,20 @@ typedef LPCSTR PCTSTR, LPCTSTR, PCUTSTR, LPCUTSTR;
 #define __attribute__(x)
 #endif
 
-#ifdef __MINGW32__
+#if defined(__MINGW32__) && (__GNUC__ < 4)
+// breaks build on MinGW gcc-4
 #ifdef __cplusplus
 extern "C" {
 #endif
 void __cdecl _fpreset (void);
 void __cdecl fpreset (void);
-#if (__GNUC__ < 4) // breaks build on MinGW gcc-4
-#define SetClassLongPtr SetClassLong
-#define GCLP_HICON GCL_HICON
-#define GCLP_HICONSM GCL_HICONSM
-#endif //GNUC
 #ifdef __cplusplus
 }
 #endif //cplusplus
-#endif //MINGW
+#define SetClassLongPtr SetClassLong
+#define GCLP_HICON GCL_HICON
+#define GCLP_HICONSM GCL_HICONSM
+#endif //MINGW32 && GNUC < 4
 
 // On the Win32 platform include file and line number information for each
 //   memory allocation/deallocation
