@@ -65,7 +65,7 @@ public:
     char virtualbox_version[256];
     // ... add entries for VMWare, others
 
-    COPROCS coprocs;
+    COPROCS _coprocs;
 
     HOST_INFO();
     int parse(MIOFILE&, bool benchmarks_only = false);
@@ -86,12 +86,6 @@ public:
     void clear_host_info();
     void make_random_string(const char* salt, char* out);
     void generate_host_cpid();
-    inline bool have_cuda() {
-        return (coprocs.cuda.count > 0);
-    }
-    inline bool have_ati() {
-        return (coprocs.ati.count > 0);
-    }
 };
 
 #ifdef __APPLE__
