@@ -266,7 +266,7 @@ bool remove_element(char* buf, const char* start, const char* end) {
     if (!p) return false;
     q = strstr(p+strlen(start), end);
     if (!q) return false;
-    strcpy(p, q+strlen(end));
+    strcpy_overlap(p, q+strlen(end));
     return true;
 }
 
@@ -283,7 +283,7 @@ bool str_replace(char* str, const char* substr, const char* replacement) {
     strcat(p, temp);
     return true;
 }
-    
+
 // if the given XML has an element of the form
 // <venue name="venue_name">
 //   ...
@@ -309,17 +309,17 @@ void extract_venue(const char* in, const char* venue_name, char* out) {
         //
         q = in;
         strcpy(out, "");
-       	while (1) {
-           	p = strstr(q, "<venue");
-           	if (!p) {
-       	        strcat(out, q);
+           while (1) {
+               p = strstr(q, "<venue");
+               if (!p) {
+                   strcat(out, q);
                 break;
             }
-           	strncat(out, q, p-q);
-           	q = strstr(p, "</venue>");
-           	if (!q) break;
-           	q += strlen("</venue>");
-       	}
+               strncat(out, q, p-q);
+               q = strstr(p, "</venue>");
+               if (!q) break;
+               q += strlen("</venue>");
+           }
     }
 }
 
@@ -469,7 +469,6 @@ int skip_unrecognized(char* buf, MIOFILE& fin) {
         if (strstr(buf2, close_tag.c_str())) {
             return 0;
         }
-        
     }
     return ERR_XML_PARSE;
 }
@@ -512,7 +511,7 @@ int XML_PARSER::scan_comment() {
             return XML_PARSE_COMMENT;
         }
         if (strlen(buf) > 32) {
-            strcpy(buf, buf+16);
+            strcpy_overlap(buf, buf+16);
             p = buf;
         }
     }
@@ -629,7 +628,7 @@ bool XML_PARSER::copy_until_tag(char* buf, int len) {
 int XML_PARSER::get_aux(char* buf, int len, char* attr_buf, int attr_len) {
     bool eof;
     int c, retval;
-    
+
     while (1) {
         eof = scan_nonws(c);
         if (eof) return XML_PARSE_EOF;
@@ -676,11 +675,14 @@ bool XML_PARSER::parse_str(
 
     // handle the archaic form <tag/>, which means empty string
     //
-    strcpy(tag, start_tag);
-    strcat(tag, "/");
-    if (!strcmp(parsed_tag, tag)) {
-        strcpy(buf, "");
-        return true;
+    int n = strlen(parsed_tag);
+    if (parsed_tag[n-1] == '/') {
+        strcpy(tag, parsed_tag);
+        tag[n-1] = 0;
+        if (!strcmp(tag, start_tag)) {
+            strcpy(buf, "");
+            return true;
+        }
     }
 
     // check for start tag
@@ -859,27 +861,27 @@ bool XML_PARSER::parse_start(const char* start_tag) {
 // strips whitespace.
 //
 int XML_PARSER::element_contents(const char* end_tag, char* buf, int buflen) {
-	int n=0;
-	int retval=0;
+    int n=0;
+    int retval=0;
     while (1) {
-		if (n == buflen-1) {
-			retval = ERR_XML_PARSE;
-			break;
-		}
+        if (n == buflen-1) {
+            retval = ERR_XML_PARSE;
+            break;
+        }
         int c = f->_getc();
-		if (c == EOF) {
-			retval = ERR_XML_PARSE;
-			break;
-		}
-		buf[n++] = c;
-		buf[n] = 0;
-		char* p = strstr(buf, end_tag);
-		if (p) {
-			*p = 0;
-			break;
-		}
+        if (c == EOF) {
+            retval = ERR_XML_PARSE;
+            break;
+        }
+        buf[n++] = c;
+        buf[n] = 0;
+        char* p = strstr(buf, end_tag);
+        if (p) {
+            *p = 0;
+            break;
+        }
     }
-	buf[n] = 0;
+    buf[n] = 0;
     strip_whitespace(buf);
     return retval;
 }
@@ -909,6 +911,24 @@ void XML_PARSER::skip_unexpected(
     }
 }
 
+// copy an element (which may contain tags) to the buffer
+//
+int XML_PARSER::element(const char* start_tag, char* buf, int buflen) {
+    char end_tag[256];
+
+    int n = strlen(start_tag);
+    if (start_tag[n-1] == '/') {
+        sprintf(buf, "<%s>", start_tag);
+        return 0;
+    }
+    if (strchr(start_tag, '/')) return ERR_XML_PARSE;
+    sprintf(end_tag, "</%s>", start_tag);
+    int retval = element_contents(end_tag, buf, buflen);
+    if (retval) return retval;
+    strcat(buf, end_tag);
+    return 0;
+}
+
 // sample use is shown below
 
 #if 0
