@@ -64,7 +64,7 @@ int delete_host_file(int host_id, const char* file_name) {
     sprintf(mth.variety, "delete_file");
     retval = mth.insert();
     if (retval) {
-        fprintf(stderr, "msg_to_host.insert(): %d\n", retval);
+        fprintf(stderr, "msg_to_host.insert(): %s\n", boincerror(retval));
         return retval;
     }
     return 0;
@@ -123,7 +123,7 @@ int main(int argc, char** argv) {
         config.db_name, config.db_host, config.db_user, config.db_passwd
     );
     if (retval) {
-        fprintf(stderr, "boinc_db.open failed: %d\n", retval);
+        fprintf(stderr, "boinc_db.open failed: %s\n", boincerror(retval));
         exit(1);
     }
 
@@ -132,4 +132,4 @@ int main(int argc, char** argv) {
     return retval;
 }
 
-const char *BOINC_RCSID_f6337b04b0 = "$Id: delete_file.cpp 21181 2010-04-15 03:13:56Z davea $";
+const char *BOINC_RCSID_f6337b04b0 = "$Id: delete_file.cpp 22647 2010-11-08 17:51:57Z davea $";
