@@ -79,12 +79,12 @@ static int send_assigned_job(ASSIGNMENT& asg) {
     retval = create_result(wu, (char *)rtfpath, suffix, key, config, 0, 0);
     if (retval) {
         log_messages.printf(MSG_CRITICAL,
-            "[WU#%d %s] create_result() %d\n", wu.id, wu.name, retval
+            "[WU#%d %s] create_result(): %s\n", wu.id, wu.name, boincerror(retval)
         );
         return retval;
     }
     int result_id = boinc_db.insert_id();
-    DB_RESULT result;
+    SCHED_DB_RESULT result;
     retval = result.lookup_id(result_id);
     add_result_to_reply(result, wu, bavp, false);
 
@@ -98,7 +98,7 @@ static int send_assigned_job(ASSIGNMENT& asg) {
         retval = db_asg.update_field(buf);
         if (retval) {
             log_messages.printf(MSG_CRITICAL,
-                "assign update failed: %d\n", retval
+                "assign update failed: %s\n", boincerror(retval)
             );
             return retval;
         }
@@ -123,6 +123,7 @@ bool send_assigned_jobs() {
     bool sent_something = false;
 
     for (int i=0; i<ssp->nassignments; i++) {
+        if (!work_needed(false)) break; 
         ASSIGNMENT& asg = ssp->assignments[i];
 
         if (config.debug_assignment) {
