@@ -71,4 +71,7 @@ extern int create_work(
     const char* additional_xml = NULL
 );
 
+extern int put_file(int host_id, const char* file_name);
+extern int get_file(int host_id, const char* file_name);
+
 #endif
