@@ -7,16 +7,16 @@
 #define BOINC_MAJOR_VERSION 6
 
 /* Minor part of BOINC version number */
-#define BOINC_MINOR_VERSION 12
+#define BOINC_MINOR_VERSION 13
 
 /* Release part of BOINC version number */
-#define BOINC_RELEASE 33
+#define BOINC_RELEASE 1
 
 /* String representation of BOINC version number */
-#define BOINC_VERSION_STRING "6.12.33"
+#define BOINC_VERSION_STRING "6.13.1"
 
 /* Package is a pre-release (Alpha/Beta) package */
-/* #define BOINC_PRERELEASE 1 */
+#define BOINC_PRERELEASE 1
 
 #if (defined(_WIN32) || defined(__APPLE__))
 /* Name of package */
@@ -29,13 +29,13 @@
 #define PACKAGE_NAME "BOINC"
 
 /* Define to the full name and version of this package. */
-#define PACKAGE_STRING "BOINC 6.12.33"
+#define PACKAGE_STRING "BOINC 6.13.1"
 
 /* Define to the one symbol short name of this package. */
 #define PACKAGE_TARNAME "boinc"
 
 /* Define to the version of this package. */
-#define PACKAGE_VERSION "6.12.33"
+#define PACKAGE_VERSION "6.13.1"
 
 #endif /* #if (defined(_WIN32) || defined(__APPLE__)) */
 
